%% Implements a fourth-order Runge-Kutta method
function out = ODE_RK4(y,dt)
k1 = ODE_func(y);
k2 = ODE_func(y + dt/2 * k1);
k3 = ODE_func(y + dt/2 * k2);
k4 = ODE_func(y + dt   * k3);

out = y + dt/6 * (k1 + 2*k2 + 2*k3 + k4);

end

%% ODE function for the lattice system
function out = ODE_func(Y)
global hf_ind
global x_ind
global v_ind
global h_four
global NU
global k
global Nr

% each variable is a column vector (as Y is a column vector)
x = Y(x_ind);       % droplet positions
v = Y(v_ind);       % droplet velocities
hf = Y(hf_ind);     % wave field in Fourier space

% gradient beneath each droplet
grad = 1/Nr * real(sum(1i * k .* hf.' .* exp(1i * x * k), 2));

% output dY/dt = F(Y)
out = zeros(size(Y));
out(x_ind) = v;
out(v_ind) = -(v + grad);
out(hf_ind) = -NU*hf + (h_four .* sum(exp(-1i * x * k), 1)).';
end